@@ -99,7 +99,7 @@ int do_debug = qtdb_gc;
 int do_debug = qtdb_none;
 #endif
 
-typedef QHash<void *, VALUE *> PointerMap;
+typedef QHash<void *, SmokeValue> PointerMap;
 Q_GLOBAL_STATIC(PointerMap, pointer_map)
 int object_count = 0;
 
@@ -149,6 +149,10 @@ void *value_to_ptr(VALUE ruby_value) {  // ptr on success, null on fail
 }
 
 VALUE getPointerObject(void *ptr) {
+  return getSmokeValue(ptr).value;
+}
+
+SmokeValue getSmokeValue(void *ptr) {
 	if (!pointer_map() || !pointer_map()->contains(ptr)) {
 		if (do_debug & qtdb_gc) {
 			qWarning("getPointerObject %p -> nil", ptr);
@@ -156,63 +160,88 @@ VALUE getPointerObject(void *ptr) {
 				qWarning("getPointerObject pointer_map deleted");
 			}
 		}
-	    return Qnil;
+	    return SmokeValue();
 	} else {
 		if (do_debug & qtdb_gc) {
-			qWarning("getPointerObject %p -> %p", ptr, (void *) *(pointer_map()->operator[](ptr)));
+			qWarning("getPointerObject %p -> %p", ptr, (void *) pointer_map()->operator[](ptr).value);
 		}
-		return *(pointer_map()->operator[](ptr));
+		return pointer_map()->operator[](ptr);
+	}
+}
+
+void unmapPointer(void *ptr, Smoke *smoke, Smoke::Index fromClassId, Smoke::Index toClassId, void *lastptr) {
+	ptr = smoke->cast(ptr, fromClassId, toClassId);
+
+	if (ptr != lastptr) {
+		lastptr = ptr;
+		if (pointer_map() && pointer_map()->contains(ptr)) {
+			VALUE obj_ptr = pointer_map()->operator[](ptr).value;
+		
+			if (do_debug & qtdb_gc) {
+				const char *className = smoke->classes[fromClassId].className;
+				qWarning("unmapPointer (%s*)%p -> %p size: %d", className, ptr, (void*)(&obj_ptr), pointer_map()->size() - 1);
+			}
+
+			pointer_map()->remove(ptr);
+		}
+	}
+
+	if (smoke->classes[toClassId].external) {
+		// encountered external class
+		Smoke::ModuleIndex mi = Smoke::findClass(smoke->classes[toClassId].className);
+		if (!mi.index || !mi.smoke) return;
+
+		smoke = mi.smoke;
+		toClassId = mi.index;
+	}
+
+	for (Smoke::Index *i = smoke->inheritanceList + smoke->classes[toClassId].parents; *i; i++) {
+		unmapPointer(ptr, smoke, toClassId, *i, lastptr);
 	}
 }
 
 void unmapPointer(smokeruby_object *o, Smoke::Index classId, void *lastptr) {
-	void *ptr = o->smoke->cast(o->ptr, o->classId, classId);
-	if (ptr != lastptr) {
-		lastptr = ptr;
-		if (pointer_map() && pointer_map()->contains(ptr)) {
-			VALUE * obj_ptr = pointer_map()->operator[](ptr);
-		
-			if (do_debug & qtdb_gc) {
-				const char *className = o->smoke->classes[o->classId].className;
-				qWarning("unmapPointer (%s*)%p -> %p size: %d", className, ptr, obj_ptr, pointer_map()->size() - 1);
-			}
-	    
-			pointer_map()->remove(ptr);
-			xfree((void*) obj_ptr);
-		}
-    }
-
-	for (Smoke::Index *i = o->smoke->inheritanceList + o->smoke->classes[classId].parents; *i; i++) {
-		unmapPointer(o, *i, lastptr);
-	}
+	unmapPointer(o->ptr, o->smoke, o->classId, classId, lastptr);
 }
 
 // Store pointer in pointer_map hash : "pointer_to_Qt_object" => weak ref to associated Ruby object
 // Recurse to store it also as casted to its parent classes.
 
-void mapPointer(VALUE obj, smokeruby_object *o, Smoke::Index classId, void *lastptr) {
-    void *ptr = o->smoke->cast(o->ptr, o->classId, classId);
-	
+void mapPointer(VALUE obj, smokeruby_object* o, void *ptr, Smoke *smoke, Smoke::Index fromClassId, Smoke::Index toClassId, void *lastptr) {
+	ptr = smoke->cast(ptr, fromClassId, toClassId);
+
     if (ptr != lastptr) {
 		lastptr = ptr;
-		VALUE * obj_ptr = ALLOC(VALUE);
-		memcpy(obj_ptr, &obj, sizeof(VALUE));
-		
+        
 		if (do_debug & qtdb_gc) {
-			const char *className = o->smoke->classes[o->classId].className;
+			const char *className = smoke->classes[fromClassId].className;
 			qWarning("mapPointer (%s*)%p -> %p size: %d", className, ptr, (void*)obj, pointer_map()->size() + 1);
 		}
-	
-		pointer_map()->insert(ptr, obj_ptr);
+        
+        SmokeValue value(obj, o);
+		pointer_map()->insert(ptr, value);
     }
-	
-	for (Smoke::Index *i = o->smoke->inheritanceList + o->smoke->classes[classId].parents; *i; i++) {
-		mapPointer(obj, o, *i, lastptr);
+
+	if (smoke->classes[toClassId].external) {
+		// encountered external class
+		Smoke::ModuleIndex mi = Smoke::findClass(smoke->classes[toClassId].className);
+		if (!mi.index || !mi.smoke) return;
+
+		smoke = mi.smoke;
+		toClassId = mi.index;
+	}
+
+	for (Smoke::Index *i = smoke->inheritanceList + smoke->classes[toClassId].parents; *i; i++) {
+		mapPointer(obj, o, ptr, smoke, toClassId, *i, lastptr);
 	}
 	
 	return;
 }
 
+void mapPointer(VALUE obj, smokeruby_object *o, Smoke::Index classId, void *lastptr) {
+	mapPointer(obj, o, o->ptr, o->smoke, o->classId, classId, lastptr);
+}
+
 namespace QtRuby {
 
 Binding::Binding() : SmokeBinding(0) {}
@@ -223,9 +252,10 @@ Binding::deleted(Smoke::Index classId, void *ptr) {
 	if (!pointer_map()) {
 	return;
 	}
-	VALUE obj = getPointerObject(ptr);
-	smokeruby_object *o = value_obj_info(obj);
+
+	smokeruby_object *o = getSmokeValue(ptr).o;
 	if (do_debug & qtdb_gc) {
+	  	qWarning("unmapping: o = %p, ptr = %p\n", o, ptr);
     	qWarning("%p->~%s()", ptr, smoke->className(classId));
     }
 	if (!o || !o->ptr) {
@@ -270,11 +300,7 @@ Binding::callMethod(Smoke::Index method, void *ptr, Smoke::Stack args, bool /*is
 	}
 		// If the virtual method hasn't been overriden, just call the C++ one.
 		// During GC, avoid checking for override and just call the C++ version
-#if RUBY_VERSION < 0x10807
-	if (rb_respond_to(obj, rb_intern(methodName)) == 0) {
-#else
 	if (rb_during_gc() || rb_respond_to(obj, rb_intern(methodName)) == 0) {
-#endif
     	return false;
 	}
 	QtRuby::VirtualMethodCall c(smoke, method, args, obj, ALLOCA_N(VALUE, smoke->methods[method].numArgs));
@@ -1037,7 +1063,7 @@ static QRegExp * rx = 0;
 			} else if (staticType == "uint") {
 				arg->argType = xmoc_uint;
 				smoke = qtcore_Smoke;
-				typeId = smoke->idType(name.constData());
+				typeId = smoke->idType("unsigned int");
 			} else if (staticType == "long") {
 				arg->argType = xmoc_long;
 				smoke = qtcore_Smoke;
@@ -1045,7 +1071,7 @@ static QRegExp * rx = 0;
 			} else if (staticType == "ulong") {
 				arg->argType = xmoc_ulong;
 				smoke = qtcore_Smoke;
-				typeId = smoke->idType(name.constData());
+				typeId = smoke->idType("unsigned long");
 			} else if (staticType == "double") {
 				arg->argType = xmoc_double;
 				smoke = qtcore_Smoke;
